@@ -2,11 +2,10 @@
 use strict;
 use warnings;
 package App::Spec::Plugin::Format;
-our $VERSION = 'v0.15.0'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 use YAML::PP;
 use Ref::Util qw/ is_arrayref /;
-use Encode;
 
 use Moo;
 with 'App::Spec::Role::Plugin::GlobalOptions';
@@ -49,21 +48,21 @@ sub print_output {
         next unless $out->type eq 'data';
         my $content = $out->content;
         if ($format eq 'YAML') {
-            $content = encode_utf8 YAML::PP::Dump($content);
+            $content = YAML::PP::Dump($content);
         }
         elsif ($format eq 'JSON') {
             require JSON::XS;
             my $coder = JSON::XS->new->ascii->pretty->allow_nonref;
-            $content = encode_utf8 $coder->encode($content) . "\n";
+            $content = $coder->encode($content) . "\n";
         }
         elsif ($format eq 'Table' and is_arrayref($content)) {
             require Text::Table;
             my $header = shift @$content;
             my $tb = Text::Table->new( @$header );
             $tb->load(@$content);
-            $content = encode_utf8 "$tb";
+            $content = "$tb";
         }
-        elsif ($format eq 'Data::Dump') {
+        elsif ($format eq 'Data__Dump') {
             require Data::Dump;
             $content = Data::Dump::dump($content) . "\n";
         }
@@ -115,5 +114,5 @@ __DATA__
 -   name: format
     summary: Format output
     type: string
-    enum: [JSON, YAML, Table, "Data::Dumper", "Data::Dump"]
+    enum: [JSON, YAML, Table, "Data__Dumper", "Data__Dump"]
 
